# Copyright (C) 2008 One Laptop Per Child
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import logging
import subprocess
from gettext import gettext as _
import errno
import time

from gi.repository import Gio
from gi.repository import GLib

from jarabe import config
from jarabe.model.network import get_wireless_interfaces


_OFW_TREE = '/ofw'
_PROC_TREE = '/proc/device-tree'
_DMI_DIRECTORY = '/sys/class/dmi/id'

_logger = logging.getLogger('ControlPanel - AboutComputer')
_not_available = _('Not available')

_serial_no = None


def get_aboutcomputer():
    msg = 'Serial Number: %s \nBuild Number: %s \nFirmware Number: %s \n' \
        % (get_serial_number(), get_build_number(), get_firmware_number())
    return msg


def print_aboutcomputer():
    print(get_aboutcomputer())


def _get_serial_number():
    serial_no = _read_device_tree('serial-number')
    if serial_no is not None:
        return serial_no

    cmd = 'pkexec sugar-serial-number-helper'
    result, output, error, status = GLib.spawn_command_line_sync(cmd)
    if status != 0:
        return _not_available

    return output.decode().rstrip('\n')


def get_serial_number():
    global _serial_no

    if _serial_no is None:
        _serial_no = _get_serial_number()

    return _serial_no


def print_serial_number():
    serial_no = get_serial_number()
    if serial_no is None:
        serial_no = _not_available
    print(serial_no)


def get_build_number():
    build_no = _read_file('/boot/olpc_build')

    if build_no is None:
        build_no = _read_file('/etc/redhat-release')

    if build_no is None:
        try:
            popen = subprocess.Popen(['lsb_release', '-ds'],
                                     stdout=subprocess.PIPE)
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise
        else:
            build_no, stderr_ = popen.communicate()
            build_no = build_no.strip().decode()

    if build_no is None or not build_no:
        build_no = _not_available

    return build_no


def print_build_number():
    print(get_build_number())


def get_firmware_number():
    firmware_no = _read_device_tree('openprom/model')
    if firmware_no is not None:
        # try to extract Open Firmware version from OLPC style version
        # string, e.g. "CL2   Q4B11  Q4B"
        if firmware_no.startswith('CL'):
            firmware_no = firmware_no[6:13].strip()
        ec_name = _read_device_tree('ec-name')
        if ec_name:
            ec_name = ec_name.replace('Ver:', '')
            firmware_no = '%(firmware)s with %(ec)s' % {
                'firmware': firmware_no, 'ec': ec_name}

    elif os.path.exists(os.path.join(_DMI_DIRECTORY, 'bios_version')):
        firmware_no = _read_file(os.path.join(_DMI_DIRECTORY, 'bios_version'))
    if firmware_no is None:
        firmware_no = _not_available
    return firmware_no


def get_hardware_model():
    settings = Gio.Settings.new('org.sugarlabs.extensions.aboutcomputer')
    model = settings.get_string('hardware-model')
    if model:
        return model

    model = _read_device_tree('mfg-data/MN')

    if model is None:
        if 'NL3' in get_firmware_number():
            model = 'NL3'
        if 'ONEDU' in get_firmware_number():
            model = 'One Education Infinity'
        if 'VirtualBox' in get_firmware_number():
            model = 'VirtualBox VM'
        cores = 0
        cpu = ''
        mhz = ''
        for line in open('/proc/cpuinfo').readlines():
            if 'model name' in line:
                cores += 1
                split = line.split(' ')
                if split[2] == 'Intel(R)' and \
                   split[3] == 'Celeron(R)':
                    if split[5] == 'CPU':  # ed20gm
                        cpu = split[4]
                        mhz = split[7].strip().replace('GHz', ' GHz')
                    if split[4] == 'CPU':  # nl3
                        cpu = split[6]
                        mhz = split[9].strip().replace('GHz', ' GHz')
        if cores == 2:
            cores = 'Dual Core '
        elif cores == 4:
            cores = 'Quad Core '
        else:
            cores = ''
        if cpu != '' and mhz != '':
            model = '%s, %sCPU %s @ %s' % (model, cores, cpu, mhz)

    meminfo = open('/proc/meminfo').readlines()
    memtotal = int(meminfo[0].split(':')[1].replace('kB', ''))
    mem = '%.0f GB' % (memtotal / 1000000.)

    # 1929056 NL3
    # 2063896 XO-4
    # 861128 XO-1.75
    # 957912 XO-1.5
    # 226012 XO-1

    if 200000 < memtotal < 256000:  # XO-1
        mem = '256 MB'
    elif 800000 < memtotal < 1200000:  # XO-1.5 XO-1.75
        mem = '1 GB'
    elif 1800000 < memtotal < 2200000:  # XO-4 NL3
        mem = '2 GB'
    model = '%s, %s RAM' % (model, mem)

    statvfs = os.statvfs('.')
    disk = statvfs.f_bsize * statvfs.f_blocks

    def near(n, size):
        a = size * 0.75
        b = size * 1.25
        return a < n < b

    if near(disk, 128000000000):  # 124959473664 ED2 D ed2-d 128 GB SATA SSD
        size = '128 GB'
    elif near(disk, 32000000000):  # 28119871488 NL3 B nl3-b 32 GB eMMC
        size = '32 GB'
    elif near(disk, 16000000000):  # 13099556864 NL3 C nl3-d 16 GB SATA SSD
        size = '16 GB'
    elif near(disk,  8000000000):  # 7755186176 XO-4 C2 SKU301 pouzl-p 8 GB
                                   # 7759708160 XO-1.75 C1 SKU201 pouzl-c 8 GB
        size = '8 GB'
    elif near(disk,  4000000000):  # 3799687168 XO-1.5 C5 SKU quozl-w 4 GB
        size = '4 GB'
    elif near(disk,  2000000000):
        size = '2 GB'
    elif near(disk,  1000000000):  # 1073741824 XO-1 SKU quozl-j 1 GB
        size = '1 GB'
    else:
        size = '%.0f GB' % (disk / 1000000000.)
    model = '%s, %s disk' % (model, size)

    sku = _read_device_tree('mfg-data/sk')
    if sku:
        if sku.startswith('SKU'):
            model = '%s, %s' % (model, sku)
        else:
            model = '%s, SKU%s' % (model, sku)

    return model


def get_secondary_licenses():
    licenses = []
    # Check if there are more licenses to display
    licenses_path = config.licenses_path
    if os.path.isdir(licenses_path):
        for file_name in os.listdir(licenses_path):
            try:
                file_path = os.path.join(licenses_path, file_name)
                with open(file_path) as f:
                    licenses.append(f.read())
            except IOError:
                logging.error('Error trying open %s', file_path)
    return licenses


def print_firmware_number():
    print(get_firmware_number())


def get_wireless_firmware():
    environment = os.environ.copy()
    environment['PATH'] = '%s:/usr/sbin' % (environment['PATH'], )
    firmware_info = {}

    wireless_interfaces = get_wireless_interfaces()
    if not wireless_interfaces:
        _logger.warning('NetworkManager knows of no wireless devices, '
                        'falling back to static list')
        wireless_interfaces = ['wlan0', 'eth0']

    for interface in get_wireless_interfaces():
        try:
            output = subprocess.Popen(['ethtool', '-i', interface],
                                      stdout=subprocess.PIPE,
                                      env=environment).stdout.readlines()
        except OSError:
            _logger.exception('Error running ethtool for %r', interface)
            continue

        try:
            for line in output:
                line = line.decode()
                if line.startswith('firmware'):
                    version = line.split()[1]
                if line.startswith('driver'):
                    driver = line.split()[1]
        except IndexError:
            _logger.exception('Error parsing ethtool output for %r',
                              interface)
            continue

        if version == 'N/A':
            version = 'Loaded'

        card = None
        if driver == 'mwifiex':
            card = 'mv8787, IEEE 802.11n 5GHz'
        if driver == 'libertas':
            if version.startswith('5.'):
                card = 'usb8388, IEEE 802.11g 2.4GHz'
            else:
                card = 'mv8686, IEEE 802.11g 2.4GHz'
        if driver == 'rtl8821ae':
            card = 'IEEE 802.11ac 5 GHz'
        if driver == 'r8822be':
            card = 'IEEE 802.11ac 5 GHz (2x2) Bluetooth 4.2'

        if card:
            firmware_info[interface] = '%s (%s, %s)' % (version, driver, card)
        else:
            firmware_info[interface] = '%s (%s)' % (version, driver)

    if not firmware_info:
        return _not_available

    if len(firmware_info) == 1:
        return list(firmware_info.values())[0]

    return ', '.join(['%(interface)s: %(info)s' %
                      {'interface': interface, 'info': info}
                      for interface, info in list(firmware_info.items())])


def print_wireless_firmware():
    print(get_wireless_firmware())


def _read_file(path):
    if os.access(path, os.R_OK) == 0:
        return None

    fd = open(path, 'r')
    value = fd.read()
    fd.close()
    if value:
        value = value.strip('\n')
        return value
    else:
        _logger.debug('No information in file or directory: %s', path)
        return None


def get_license():
    license_file = os.path.join(config.data_path, 'GPLv3')
    lang = os.environ['LANG']
    if lang.endswith('UTF-8'):
        lang = lang[:-6]

    try_file = license_file + '.' + lang
    if os.path.isfile(try_file):
        license_file = try_file
    else:
        try_file = license_file + '.' + lang.split('_')[0]
        if os.path.isfile(try_file):
            license_file = try_file

    try:
        fd = open(license_file)
        # remove 0x0c page breaks which can't be rendered in text views
        license_text = fd.read().replace('\x0c', '')
        fd.close()
    except IOError:
        license_text = _not_available
    return license_text


def _read_device_tree(path):
    value = _read_file(os.path.join(_PROC_TREE, path))
    if value:
        return value.strip('\x00')
    value = _read_file(os.path.join(_OFW_TREE, path))
    if value:
        return value.strip('\x00')
    return value


def days_from_last_update():

    last_update_seconds = -1
    # Get the number of seconds of the last update date.
    try:
        flag_file = '/var/lib/misc/last_os_update.stamp'
        if os.path.exists(flag_file):
            last_update_seconds = int(os.stat(flag_file).st_mtime)
    except IOError:
        _logger.error('couldn''t get last modification time')

    if last_update_seconds == -1:
        return -1

    now = time.time()
    days_from_last_update = (now - last_update_seconds) / (24 * 60 * 60)
    return int(days_from_last_update)
